'''
Data Loader
-Load chips, animations, maps, sounds, and high score.
-Save the high score.
'''

import os
import ast
import pygame
from pygame.locals import *

class Loader():
    def __init__(self, last_stage):
        self.data_py = os.path.abspath(os.path.dirname(__file__))
        self.data_dir = os.path.normpath(os.path.join(self.data_py, '..', 'data'))

        #Chip animation loading
        self.anime_dict = {}
        f = self.load('animation/animation.dat', 'r')
        lines = f.readlines()
        f.close()
        
        self.anime_dict = ast.literal_eval(lines[0])
        for i in self.anime_dict.items():
            chip_pre = i[0]
            chip_number = i[1]
            for t in range(0, chip_number[0]):
                k = "0"+str(t)
                chip_name = chip_pre+k[-2:]+".png"
                transparent = -1

                self.anime_dict[chip_pre].append(self.load_image("animation/"+chip_name, transparent))

        self.picture_dict = {"title":"title.png", "game_over":"game_over.png"}

        #print (self.anime_dict)
        print ("Animation loading is done.")

        #Sound
        self.sound_table = {"coin":"coin.wav", "jet":"jet.wav", "exit":"exit.wav", "faint":"faint.wav", "thud":"thud.wav", "yelp":"yelp.wav"}
        self.sound_dict = {}
        for temp in self.sound_table.items():
            self.sound_dict[temp[0]] = self.load_sound(temp[1])
            self.sound_dict[temp[0]].set_volume(0.1)
    
        #Gave up playing BGM. self.bgm_dict = {"main":"***.mp3"}
        print ("Sound loading is done.")

        #Map: reading
        self.map_list = []
        self.map_file = []
        for i in range(0, last_stage):
            f = self.load('map/stage'+str(i+1)+'.map', 'r')
            lines = f.readlines()
            f.close()

            #Map: chip number list
            self.map_list.append(ast.literal_eval(lines[0].rstrip()))

            #Map: chip number and related file
            temp = ast.literal_eval(lines[1].rstrip())
            self.map_file.append(dict((v,k) for k,v in temp.items())) #Swap key and value

        #Map: loading tile images
        self.map_image = {}
        for i in range(0, 84):
            k = "0"+str(i)
            chip_name = "out"+k[-2:]+".png"
            self.map_image[chip_name] = self.load_image("tile/"+chip_name)

        #Map: filename(ID) and item name
        self.map_item = {"out00.png":"exit", "out01.png":"entrance", "out04.png":"coin_copper", "out20.png":"floor", "out42.png":"wall", "out43.png":"coin_silver", "out44.png":"coin_gold"}
        print ("Map loading is done.")
        
        #Font loading
        temp = "hand_pixelrock.ttf"
        path = self.filepath("font/"+temp)
        self.font_main = pygame.font.Font(path, 50)
        temp = "digital_counter_7_italic.ttf"
        path = self.filepath("font/"+temp)
        self.font_counter = pygame.font.Font(path, 420)
        self.font_stage = pygame.font.Font(path, 150) #Bad coding
        self.font_jet = pygame.font.Font(path, 30) #Bad coding

        print ("Font loading is done.")

    def load_picture(self, name):
        path = self.filepath("picture/"+self.picture_dict[name])
        return pygame.image.load(path)
        
    def load_music(self, name):
        path = self.filepath("sound/"+self.bgm_dict[name])
        pygame.mixer.music.load(path)

    def load_sound(self, name):
        path = self.filepath("sound/"+name)
        return pygame.mixer.Sound(path)

    def get_floor_image(self):
        #Workaround, really bad code
        return self.map_image["out20.png"]

    def get_map_image(self, id, stage):
        #Return an image object corresponding to the chip number
        #Each stage has different chip number order, because of the MAP editor I used
        temp_dict = self.map_file[stage-1]
        temp_name = temp_dict[id]
        return self.map_image[temp_name]
    
    def get_map_role(self, id, stage):
        #Return a map role corresponding to the chip number
        temp_dict = self.map_file[stage-1]
        temp_name = temp_dict[id]
        if temp_name in self.map_item:
            return self.map_item[temp_name]
        return False

    def load_image(self, name, colorkey=None):
        fullname = self.filepath(name)
        image = pygame.image.load(fullname)

        image = image.convert()
        if colorkey is not None:
            if colorkey is -1:
                colorkey = image.get_at((0, 0))
            image.set_colorkey(colorkey, RLEACCEL)
        return image
                
    def filepath(self, filename):
        return os.path.join(self.data_dir, filename)
    
    def load(self, filename, mode='rb'):
        return open(os.path.join(self.data_dir, filename), mode)
    
    def get_high_score(self):
        #Gave up high score update
        return 1000

    def save_high_score(self):
        #Gave up high score update
        pass
