'''
System Manager
-Loading & Saving
-System Flow Control
-Title, Game Over, Ending Sequence
'''

import pygame
from pygame.locals import *

from . import game
from . import data

FPS = 60
SCREEN_WIDTH = 1024
SCREEN_HEIGHT = 768

#For the workaround of sound delay issue
pygame.mixer.pre_init(44100, -16, 2, 512)
pygame.mixer.init()

pygame.init()
SCREEN = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))
pygame.display.set_caption('Win $$$ Land')
pygame.mouse.set_visible(0)

LAST_STAGE = 5
LOADER = data.Loader(LAST_STAGE)
DEFAULT_LIFE = 3
DEFAULT_TIME = 5
GAME_OBJ = game.GameManager(SCREEN, FPS, LOADER, LAST_STAGE, DEFAULT_LIFE, DEFAULT_TIME)

def title():
    global SCREEN, LOADER
    temp_image = LOADER.load_picture("title")
    SCREEN.blit(temp_image,  [0, 0])
    pygame.display.update()
    ret = input_wait_sub()
    return ret

def ending():
    global SCREEN, LOADER, GAME_OBJ
    SCREEN.fill([255, 255, 255])

    #Change the life stock to 100 pts
    GAME_OBJ.score = GAME_OBJ.score + GAME_OBJ.life_obj.life_stock * 100 #Bad code
    font = LOADER.font_main #Bad code
    text = font.render("After being pummeled by the chimps, you Won " + str(GAME_OBJ.score) + " $$$.", 1, (192, 64, 64))
    textpos = text.get_rect(centerx=SCREEN.get_width() / 2, centery=SCREEN.get_height() / 2)
    SCREEN.blit(text, textpos)

    text = font.render("Thank you for playing!", 1, (64, 64, 64))
    textpos = text.get_rect(centerx=SCREEN.get_width() / 2, centery=SCREEN.get_height() / 2+64)
    SCREEN.blit(text, textpos)

    pygame.display.update()
    ret = input_wait_sub()
    return ret
   
def game_over():
    global SCREEN, LOADER
    temp_image = LOADER.load_picture("game_over")
    SCREEN.blit(temp_image,  [0, 0])
    pygame.display.update()
    ret = input_wait_sub()
    return ret

def input_wait_sub():
    global SCREEN, FPS

    temp_clock = pygame.time.Clock()
    input_flag = False
    while True:
        temp_clock.tick(FPS)
        for event in pygame.event.get():
            if event.type == QUIT:
                return -1
            elif event.type == KEYDOWN and event.key == K_ESCAPE:
                return -1
            elif event.type == MOUSEBUTTONDOWN:
                input_flag = True
            elif event.type == MOUSEBUTTONUP:
                if input_flag == True:
                    return 0
    
def main():
    global GAME_OBJ, LOADER
    ret = 0
    going = True
    while going:
        start = True
        if title() != 0:
            #Quit
            going = False
            break
        while 1:
            if start == True:
                ret = GAME_OBJ.start()
                start = False
            else:
                ret = GAME_OBJ.restart()
            if ret == -1:
                #Quit
                going = False
                break
            elif ret == 1:
                if GAME_OBJ.update_high_score() == 1:
                    LOADER.save_high_score(GAME_OBJ.score)
                game_over()
                GAME_OBJ.reset()
                break
            elif ret == 2:
                if GAME_OBJ.update_high_score() == 1:
                        LOADER.save_high_score(GAME_OBJ.score)
                ending()
                GAME_OBJ.reset()
                break

    pygame.quit()

if __name__ == "__main__":
    main()

